import { BigInteger as x } from "../jsbn/jsbn.js";
import { YAHOO as l } from "./yahoo.js";
/**
 * @fileOverview
 * @name asn1-1.0.js
 * @author Kenji Urushima kenji.urushima@gmail.com
 * @version asn1 1.0.13 (2017-Jun-02)
 * @since jsrsasign 2.1
 * @license <a href="https://kjur.github.io/jsrsasign/license/">MIT License</a>
 */
var e = {};
(typeof e.asn1 > "u" || !e.asn1) && (e.asn1 = {});
e.asn1.ASN1Util = new function() {
  this.integerToByteHex = function(t) {
    var i = t.toString(16);
    return i.length % 2 == 1 && (i = "0" + i), i;
  }, this.bigIntToMinTwosComplementsHex = function(t) {
    var i = t.toString(16);
    if (i.substr(0, 1) != "-")
      i.length % 2 == 1 ? i = "0" + i : i.match(/^[0-7]/) || (i = "00" + i);
    else {
      var n = i.substr(1), r = n.length;
      r % 2 == 1 ? r += 1 : i.match(/^[0-7]/) || (r += 2);
      for (var s = "", u = 0; u < r; u++)
        s += "f";
      var f = new x(s, 16), o = f.xor(t).add(x.ONE);
      i = o.toString(16).replace(/^-/, "");
    }
    return i;
  }, this.getPEMStringFromHex = function(t, i) {
    return hextopem(t, i);
  }, this.newObject = function(t) {
    var i = e, n = i.asn1, r = n.DERBoolean, s = n.DERInteger, u = n.DERBitString, f = n.DEROctetString, o = n.DERNull, c = n.DERObjectIdentifier, a = n.DEREnumerated, d = n.DERUTF8String, R = n.DERNumericString, S = n.DERPrintableString, g = n.DERTeletexString, D = n.DERIA5String, w = n.DERUTCTime, B = n.DERGeneralizedTime, H = n.DERSequence, I = n.DERSet, j = n.DERTaggedObject, T = n.ASN1Util.newObject, O = Object.keys(t);
    if (O.length != 1)
      throw "key of param shall be only one.";
    var h = O[0];
    if (":bool:int:bitstr:octstr:null:oid:enum:utf8str:numstr:prnstr:telstr:ia5str:utctime:gentime:seq:set:tag:".indexOf(":" + h + ":") == -1)
      throw "undefined key: " + h;
    if (h == "bool")
      return new r(t[h]);
    if (h == "int")
      return new s(t[h]);
    if (h == "bitstr")
      return new u(t[h]);
    if (h == "octstr")
      return new f(t[h]);
    if (h == "null")
      return new o(t[h]);
    if (h == "oid")
      return new c(t[h]);
    if (h == "enum")
      return new a(t[h]);
    if (h == "utf8str")
      return new d(t[h]);
    if (h == "numstr")
      return new R(t[h]);
    if (h == "prnstr")
      return new S(t[h]);
    if (h == "telstr")
      return new g(t[h]);
    if (h == "ia5str")
      return new D(t[h]);
    if (h == "utctime")
      return new w(t[h]);
    if (h == "gentime")
      return new B(t[h]);
    if (h == "seq") {
      for (var b = t[h], V = [], v = 0; v < b.length; v++) {
        var A = T(b[v]);
        V.push(A);
      }
      return new H({ array: V });
    }
    if (h == "set") {
      for (var b = t[h], V = [], v = 0; v < b.length; v++) {
        var A = T(b[v]);
        V.push(A);
      }
      return new I({ array: V });
    }
    if (h == "tag") {
      var E = t[h];
      if (Object.prototype.toString.call(E) === "[object Array]" && E.length == 3) {
        var N = T(E[2]);
        return new j({
          tag: E[0],
          explicit: E[1],
          obj: N
        });
      } else {
        var y = {};
        if (E.explicit !== void 0 && (y.explicit = E.explicit), E.tag !== void 0 && (y.tag = E.tag), E.obj === void 0)
          throw "obj shall be specified for 'tag'.";
        return y.obj = T(E.obj), new j(y);
      }
    }
  }, this.jsonToASN1HEX = function(t) {
    var i = this.newObject(t);
    return i.getEncodedHex();
  };
}();
e.asn1.ASN1Util.oidHexToInt = function(t) {
  for (var s = "", i = parseInt(t.substr(0, 2), 16), n = Math.floor(i / 40), r = i % 40, s = n + "." + r, u = "", f = 2; f < t.length; f += 2) {
    var o = parseInt(t.substr(f, 2), 16), c = ("00000000" + o.toString(2)).slice(-8);
    if (u = u + c.substr(1, 7), c.substr(0, 1) == "0") {
      var a = new x(u, 2);
      s = s + "." + a.toString(10), u = "";
    }
  }
  return s;
};
e.asn1.ASN1Util.oidIntToHex = function(t) {
  var i = function(o) {
    var c = o.toString(16);
    return c.length == 1 && (c = "0" + c), c;
  }, n = function(o) {
    var c = "", a = new x(o, 10), d = a.toString(2), R = 7 - d.length % 7;
    R == 7 && (R = 0);
    for (var S = "", g = 0; g < R; g++)
      S += "0";
    d = S + d;
    for (var g = 0; g < d.length - 1; g += 7) {
      var D = d.substr(g, 7);
      g != d.length - 7 && (D = "1" + D), c += i(parseInt(D, 2));
    }
    return c;
  };
  if (!t.match(/^[0-9.]+$/))
    throw "malformed oid string: " + t;
  var r = "", s = t.split("."), u = parseInt(s[0]) * 40 + parseInt(s[1]);
  r += i(u), s.splice(0, 2);
  for (var f = 0; f < s.length; f++)
    r += n(s[f]);
  return r;
};
e.asn1.ASN1Object = function() {
  var t = "";
  this.getLengthHexFromValue = function() {
    if (typeof this.hV > "u" || this.hV == null)
      throw "this.hV is null or undefined.";
    if (this.hV.length % 2 == 1)
      throw "value hex must be even length: n=" + t.length + ",v=" + this.hV;
    var i = this.hV.length / 2, n = i.toString(16);
    if (n.length % 2 == 1 && (n = "0" + n), i < 128)
      return n;
    var r = n.length / 2;
    if (r > 15)
      throw "ASN.1 length too long to represent by 8x: n = " + i.toString(16);
    var s = 128 + r;
    return s.toString(16) + n;
  }, this.getEncodedHex = function() {
    return (this.hTLV == null || this.isModified) && (this.hV = this.getFreshValueHex(), this.hL = this.getLengthHexFromValue(), this.hTLV = this.hT + this.hL + this.hV, this.isModified = !1), this.hTLV;
  }, this.getValueHex = function() {
    return this.getEncodedHex(), this.hV;
  }, this.getFreshValueHex = function() {
    return "";
  };
};
e.asn1.DERAbstractString = function(t) {
  e.asn1.DERAbstractString.superclass.constructor.call(this), this.getString = function() {
    return this.s;
  }, this.setString = function(i) {
    this.hTLV = null, this.isModified = !0, this.s = i, this.hV = stohex(this.s);
  }, this.setStringHex = function(i) {
    this.hTLV = null, this.isModified = !0, this.s = null, this.hV = i;
  }, this.getFreshValueHex = function() {
    return this.hV;
  }, typeof t < "u" && (typeof t == "string" ? this.setString(t) : typeof t.str < "u" ? this.setString(t.str) : typeof t.hex < "u" && this.setStringHex(t.hex));
};
l.lang.extend(e.asn1.DERAbstractString, e.asn1.ASN1Object);
e.asn1.DERAbstractTime = function(t) {
  e.asn1.DERAbstractTime.superclass.constructor.call(this), this.localDateToUTC = function(i) {
    utc = i.getTime() + i.getTimezoneOffset() * 6e4;
    var n = new Date(utc);
    return n;
  }, this.formatDate = function(i, n, r) {
    var s = this.zeroPadding, u = this.localDateToUTC(i), f = String(u.getFullYear());
    n == "utc" && (f = f.substr(2, 2));
    var o = s(String(u.getMonth() + 1), 2), c = s(String(u.getDate()), 2), a = s(String(u.getHours()), 2), d = s(String(u.getMinutes()), 2), R = s(String(u.getSeconds()), 2), S = f + o + c + a + d + R;
    if (r === !0) {
      var g = u.getMilliseconds();
      if (g != 0) {
        var D = s(String(g), 3);
        D = D.replace(/[0]+$/, ""), S = S + "." + D;
      }
    }
    return S + "Z";
  }, this.zeroPadding = function(i, n) {
    return i.length >= n ? i : new Array(n - i.length + 1).join("0") + i;
  }, this.getString = function() {
    return this.s;
  }, this.setString = function(i) {
    this.hTLV = null, this.isModified = !0, this.s = i, this.hV = stohex(i);
  }, this.setByDateValue = function(i, n, r, s, u, f) {
    var o = new Date(Date.UTC(i, n - 1, r, s, u, f, 0));
    this.setByDate(o);
  }, this.getFreshValueHex = function() {
    return this.hV;
  };
};
l.lang.extend(e.asn1.DERAbstractTime, e.asn1.ASN1Object);
e.asn1.DERAbstractStructured = function(t) {
  e.asn1.DERAbstractString.superclass.constructor.call(this), this.setByASN1ObjectArray = function(i) {
    this.hTLV = null, this.isModified = !0, this.asn1Array = i;
  }, this.appendASN1Object = function(i) {
    this.hTLV = null, this.isModified = !0, this.asn1Array.push(i);
  }, this.asn1Array = new Array(), typeof t < "u" && typeof t.array < "u" && (this.asn1Array = t.array);
};
l.lang.extend(e.asn1.DERAbstractStructured, e.asn1.ASN1Object);
e.asn1.DERBoolean = function() {
  e.asn1.DERBoolean.superclass.constructor.call(this), this.hT = "01", this.hTLV = "0101ff";
};
l.lang.extend(e.asn1.DERBoolean, e.asn1.ASN1Object);
e.asn1.DERInteger = function(t) {
  e.asn1.DERInteger.superclass.constructor.call(this), this.hT = "02", this.setByBigInteger = function(i) {
    this.hTLV = null, this.isModified = !0, this.hV = e.asn1.ASN1Util.bigIntToMinTwosComplementsHex(i);
  }, this.setByInteger = function(i) {
    var n = new x(String(i), 10);
    this.setByBigInteger(n);
  }, this.setValueHex = function(i) {
    this.hV = i;
  }, this.getFreshValueHex = function() {
    return this.hV;
  }, typeof t < "u" && (typeof t.bigint < "u" ? this.setByBigInteger(t.bigint) : typeof t.int < "u" ? this.setByInteger(t.int) : typeof t == "number" ? this.setByInteger(t) : typeof t.hex < "u" && this.setValueHex(t.hex));
};
l.lang.extend(e.asn1.DERInteger, e.asn1.ASN1Object);
e.asn1.DERBitString = function(t) {
  if (t !== void 0 && typeof t.obj < "u") {
    var i = e.asn1.ASN1Util.newObject(t.obj);
    t.hex = "00" + i.getEncodedHex();
  }
  e.asn1.DERBitString.superclass.constructor.call(this), this.hT = "03", this.setHexValueIncludingUnusedBits = function(n) {
    this.hTLV = null, this.isModified = !0, this.hV = n;
  }, this.setUnusedBitsAndHexValue = function(n, r) {
    if (n < 0 || 7 < n)
      throw "unused bits shall be from 0 to 7: u = " + n;
    var s = "0" + n;
    this.hTLV = null, this.isModified = !0, this.hV = s + r;
  }, this.setByBinaryString = function(n) {
    n = n.replace(/0+$/, "");
    var r = 8 - n.length % 8;
    r == 8 && (r = 0);
    for (var s = 0; s <= r; s++)
      n += "0";
    for (var u = "", s = 0; s < n.length - 1; s += 8) {
      var f = n.substr(s, 8), o = parseInt(f, 2).toString(16);
      o.length == 1 && (o = "0" + o), u += o;
    }
    this.hTLV = null, this.isModified = !0, this.hV = "0" + r + u;
  }, this.setByBooleanArray = function(n) {
    for (var r = "", s = 0; s < n.length; s++)
      n[s] == !0 ? r += "1" : r += "0";
    this.setByBinaryString(r);
  }, this.newFalseArray = function(n) {
    for (var r = new Array(n), s = 0; s < n; s++)
      r[s] = !1;
    return r;
  }, this.getFreshValueHex = function() {
    return this.hV;
  }, typeof t < "u" && (typeof t == "string" && t.toLowerCase().match(/^[0-9a-f]+$/) ? this.setHexValueIncludingUnusedBits(t) : typeof t.hex < "u" ? this.setHexValueIncludingUnusedBits(t.hex) : typeof t.bin < "u" ? this.setByBinaryString(t.bin) : typeof t.array < "u" && this.setByBooleanArray(t.array));
};
l.lang.extend(e.asn1.DERBitString, e.asn1.ASN1Object);
e.asn1.DEROctetString = function(t) {
  if (t !== void 0 && typeof t.obj < "u") {
    var i = e.asn1.ASN1Util.newObject(t.obj);
    t.hex = i.getEncodedHex();
  }
  e.asn1.DEROctetString.superclass.constructor.call(this, t), this.hT = "04";
};
l.lang.extend(e.asn1.DEROctetString, e.asn1.DERAbstractString);
e.asn1.DERNull = function() {
  e.asn1.DERNull.superclass.constructor.call(this), this.hT = "05", this.hTLV = "0500";
};
l.lang.extend(e.asn1.DERNull, e.asn1.ASN1Object);
e.asn1.DERObjectIdentifier = function(t) {
  var i = function(r) {
    var s = r.toString(16);
    return s.length == 1 && (s = "0" + s), s;
  }, n = function(r) {
    var s = "", u = new x(r, 10), f = u.toString(2), o = 7 - f.length % 7;
    o == 7 && (o = 0);
    for (var c = "", a = 0; a < o; a++)
      c += "0";
    f = c + f;
    for (var a = 0; a < f.length - 1; a += 7) {
      var d = f.substr(a, 7);
      a != f.length - 7 && (d = "1" + d), s += i(parseInt(d, 2));
    }
    return s;
  };
  e.asn1.DERObjectIdentifier.superclass.constructor.call(this), this.hT = "06", this.setValueHex = function(r) {
    this.hTLV = null, this.isModified = !0, this.s = null, this.hV = r;
  }, this.setValueOidString = function(r) {
    if (!r.match(/^[0-9.]+$/))
      throw "malformed oid string: " + r;
    var s = "", u = r.split("."), f = parseInt(u[0]) * 40 + parseInt(u[1]);
    s += i(f), u.splice(0, 2);
    for (var o = 0; o < u.length; o++)
      s += n(u[o]);
    this.hTLV = null, this.isModified = !0, this.s = null, this.hV = s;
  }, this.setValueName = function(r) {
    var s = e.asn1.x509.OID.name2oid(r);
    if (s !== "")
      this.setValueOidString(s);
    else
      throw "DERObjectIdentifier oidName undefined: " + r;
  }, this.getFreshValueHex = function() {
    return this.hV;
  }, t !== void 0 && (typeof t == "string" ? t.match(/^[0-2].[0-9.]+$/) ? this.setValueOidString(t) : this.setValueName(t) : t.oid !== void 0 ? this.setValueOidString(t.oid) : t.hex !== void 0 ? this.setValueHex(t.hex) : t.name !== void 0 && this.setValueName(t.name));
};
l.lang.extend(e.asn1.DERObjectIdentifier, e.asn1.ASN1Object);
e.asn1.DEREnumerated = function(t) {
  e.asn1.DEREnumerated.superclass.constructor.call(this), this.hT = "0a", this.setByBigInteger = function(i) {
    this.hTLV = null, this.isModified = !0, this.hV = e.asn1.ASN1Util.bigIntToMinTwosComplementsHex(i);
  }, this.setByInteger = function(i) {
    var n = new x(String(i), 10);
    this.setByBigInteger(n);
  }, this.setValueHex = function(i) {
    this.hV = i;
  }, this.getFreshValueHex = function() {
    return this.hV;
  }, typeof t < "u" && (typeof t.int < "u" ? this.setByInteger(t.int) : typeof t == "number" ? this.setByInteger(t) : typeof t.hex < "u" && this.setValueHex(t.hex));
};
l.lang.extend(e.asn1.DEREnumerated, e.asn1.ASN1Object);
e.asn1.DERUTF8String = function(t) {
  e.asn1.DERUTF8String.superclass.constructor.call(this, t), this.hT = "0c";
};
l.lang.extend(e.asn1.DERUTF8String, e.asn1.DERAbstractString);
e.asn1.DERNumericString = function(t) {
  e.asn1.DERNumericString.superclass.constructor.call(this, t), this.hT = "12";
};
l.lang.extend(e.asn1.DERNumericString, e.asn1.DERAbstractString);
e.asn1.DERPrintableString = function(t) {
  e.asn1.DERPrintableString.superclass.constructor.call(this, t), this.hT = "13";
};
l.lang.extend(e.asn1.DERPrintableString, e.asn1.DERAbstractString);
e.asn1.DERTeletexString = function(t) {
  e.asn1.DERTeletexString.superclass.constructor.call(this, t), this.hT = "14";
};
l.lang.extend(e.asn1.DERTeletexString, e.asn1.DERAbstractString);
e.asn1.DERIA5String = function(t) {
  e.asn1.DERIA5String.superclass.constructor.call(this, t), this.hT = "16";
};
l.lang.extend(e.asn1.DERIA5String, e.asn1.DERAbstractString);
e.asn1.DERUTCTime = function(t) {
  e.asn1.DERUTCTime.superclass.constructor.call(this, t), this.hT = "17", this.setByDate = function(i) {
    this.hTLV = null, this.isModified = !0, this.date = i, this.s = this.formatDate(this.date, "utc"), this.hV = stohex(this.s);
  }, this.getFreshValueHex = function() {
    return typeof this.date > "u" && typeof this.s > "u" && (this.date = new Date(), this.s = this.formatDate(this.date, "utc"), this.hV = stohex(this.s)), this.hV;
  }, t !== void 0 && (t.str !== void 0 ? this.setString(t.str) : typeof t == "string" && t.match(/^[0-9]{12}Z$/) ? this.setString(t) : t.hex !== void 0 ? this.setStringHex(t.hex) : t.date !== void 0 && this.setByDate(t.date));
};
l.lang.extend(e.asn1.DERUTCTime, e.asn1.DERAbstractTime);
e.asn1.DERGeneralizedTime = function(t) {
  e.asn1.DERGeneralizedTime.superclass.constructor.call(this, t), this.hT = "18", this.withMillis = !1, this.setByDate = function(i) {
    this.hTLV = null, this.isModified = !0, this.date = i, this.s = this.formatDate(this.date, "gen", this.withMillis), this.hV = stohex(this.s);
  }, this.getFreshValueHex = function() {
    return this.date === void 0 && this.s === void 0 && (this.date = new Date(), this.s = this.formatDate(this.date, "gen", this.withMillis), this.hV = stohex(this.s)), this.hV;
  }, t !== void 0 && (t.str !== void 0 ? this.setString(t.str) : typeof t == "string" && t.match(/^[0-9]{14}Z$/) ? this.setString(t) : t.hex !== void 0 ? this.setStringHex(t.hex) : t.date !== void 0 && this.setByDate(t.date), t.millis === !0 && (this.withMillis = !0));
};
l.lang.extend(e.asn1.DERGeneralizedTime, e.asn1.DERAbstractTime);
e.asn1.DERSequence = function(t) {
  e.asn1.DERSequence.superclass.constructor.call(this, t), this.hT = "30", this.getFreshValueHex = function() {
    for (var i = "", n = 0; n < this.asn1Array.length; n++) {
      var r = this.asn1Array[n];
      i += r.getEncodedHex();
    }
    return this.hV = i, this.hV;
  };
};
l.lang.extend(e.asn1.DERSequence, e.asn1.DERAbstractStructured);
e.asn1.DERSet = function(t) {
  e.asn1.DERSet.superclass.constructor.call(this, t), this.hT = "31", this.sortFlag = !0, this.getFreshValueHex = function() {
    for (var i = new Array(), n = 0; n < this.asn1Array.length; n++) {
      var r = this.asn1Array[n];
      i.push(r.getEncodedHex());
    }
    return this.sortFlag == !0 && i.sort(), this.hV = i.join(""), this.hV;
  }, typeof t < "u" && typeof t.sortflag < "u" && t.sortflag == !1 && (this.sortFlag = !1);
};
l.lang.extend(e.asn1.DERSet, e.asn1.DERAbstractStructured);
e.asn1.DERTaggedObject = function(t) {
  e.asn1.DERTaggedObject.superclass.constructor.call(this), this.hT = "a0", this.hV = "", this.isExplicit = !0, this.asn1Object = null, this.setASN1Object = function(i, n, r) {
    this.hT = n, this.isExplicit = i, this.asn1Object = r, this.isExplicit ? (this.hV = this.asn1Object.getEncodedHex(), this.hTLV = null, this.isModified = !0) : (this.hV = null, this.hTLV = r.getEncodedHex(), this.hTLV = this.hTLV.replace(/^../, n), this.isModified = !1);
  }, this.getFreshValueHex = function() {
    return this.hV;
  }, typeof t < "u" && (typeof t.tag < "u" && (this.hT = t.tag), typeof t.explicit < "u" && (this.isExplicit = t.explicit), typeof t.obj < "u" && (this.asn1Object = t.obj, this.setASN1Object(this.isExplicit, this.hT, this.asn1Object)));
};
l.lang.extend(e.asn1.DERTaggedObject, e.asn1.ASN1Object);
export {
  e as KJUR
};
